"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.command = void 0;
const command_1 = require("../command");
const projectUtils_1 = require("../projectUtils");
const error_1 = require("../error");
const logger_1 = require("../logger");
const utils_1 = require("../utils");
const apphosting = require("../gcp/apphosting");
const Table = require("cli-table");
const COLUMN_LENGTH = 20;
const TABLE_HEAD = ["Backend Id", "Repository", "Location", "URL", "Created Date", "Updated Date"];
exports.command = new command_1.Command("apphosting:backends:get <backendId>")
    .description("Get backend details of a Firebase project")
    .option("-l, --location <location>", "App Backend location", "-")
    .before(apphosting.ensureApiEnabled)
    .action(async (backendId, options) => {
    const projectId = (0, projectUtils_1.needProjectId)(options);
    const location = options.location;
    let backendsList = [];
    const table = new Table({
        head: TABLE_HEAD,
        style: { head: ["green"] },
    });
    table.colWidths = COLUMN_LENGTH;
    try {
        if (location !== "-") {
            const backendInRegion = await apphosting.getBackend(projectId, location, backendId);
            backendsList.push(backendInRegion);
            populateTable(backendInRegion, table);
        }
        else {
            const resp = await apphosting.listBackends(projectId, "-");
            const allBackends = resp.backends || [];
            backendsList = allBackends.filter((bkd) => bkd.name.split("/").pop() === backendId);
            backendsList.forEach((bkd) => populateTable(bkd, table));
        }
    }
    catch (err) {
        throw new error_1.FirebaseError(`Failed to get backend: ${backendId}. Please check the parameters you have provided.`, { original: err });
    }
    if (backendsList.length === 0) {
        (0, utils_1.logWarning)(`Found no backend with id: ${backendId}`);
        return;
    }
    logger_1.logger.info(table.toString());
    return backendsList[0];
});
function populateTable(backend, table) {
    var _a;
    const [location, , backendId] = backend.name.split("/").slice(3, 6);
    const entry = [
        backendId,
        (_a = backend.codebase.repository) === null || _a === void 0 ? void 0 : _a.split("/").pop(),
        location,
        backend.uri,
        backend.createTime,
        backend.updateTime,
    ];
    const newRow = entry.map((name) => {
        const maxCellWidth = COLUMN_LENGTH - 2;
        const chunks = [];
        for (let i = 0; name && i < name.length; i += maxCellWidth) {
            chunks.push(name.substring(i, i + maxCellWidth));
        }
        return chunks.join("\n");
    });
    table.push(newRow);
}
